@@ -36,6 +36,7 @@
 
 #include "pa_tnc/pa_tnc_attr_manager.h"
 #include "imv/imv_database.h"
+#include "imv/imv_session_manager.h"
 
 #include <library.h>
 
@@ -62,4 +63,9 @@ extern pa_tnc_attr_manager_t* imcv_pa_tnc_attributes;
  */
 extern imv_database_t* imcv_db;
 
+/**
+ * Global IMV session manager
+ */
+extern imv_session_manager_t* imcv_sessions;
+
 #endif /** IMCV_H_ @}*/
