@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2012 Andreas Steffen
+ * Copyright (C) 2011-2014 Andreas Steffen
  * HSR Hochschule fuer Technik Rapperswil
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -26,6 +26,8 @@ typedef struct pa_tnc_attr_t pa_tnc_attr_t;
 #include <library.h>
 #include <pen/pen.h>
 
+#define PA_TNC_ATTR_HEADER_SIZE		12
+
 /**
  * Interface for an RFC 5792 PA-TNC Posture Attribute.
  *
@@ -71,7 +73,7 @@ struct pa_tnc_attr_t {
 	 * @param					relative error offset within attribute body
 	 * @return					result status
 	 */
-	status_t (*process)(pa_tnc_attr_t *this, u_int32_t *offset);
+	status_t (*process)(pa_tnc_attr_t *this, uint32_t *offset);
 
 	/**
 	 * Get a new reference to the PA-TNC attribute
